import {
  ApplicationRef,
  ComponentFactory,
  ComponentFactoryResolver,
  ComponentRef,
  Directive,
  ElementRef,
  Injector,
  Input,
  ViewContainerRef
} from '@angular/core';
import {ViewLoadingComponent} from './loading.component';

@Directive({
  selector: '[viewLoading]'
})
export class ViewLoadingDirective {
  @Input() size: 'small' | 'middle' | 'large' = 'middle';
  @Input() delay = 0; // 等待时间

  private component: ComponentRef<ViewLoadingComponent>;
  private delayTimer: any; // 等待事件

  constructor(
    private elementRef: ElementRef,
    private viewContainerRef: ViewContainerRef,
    private injector: Injector,
    private applicationRef: ApplicationRef,
    private componentFactoryResolver: ComponentFactoryResolver
  ) {
  }

  @Input('viewLoading') set loading(isLoading: boolean) {
    if (isLoading) {
      this.delayTimer = setTimeout(() => {
        // tslint:disable-next-line:max-line-length
        const componentComponentFactory: ComponentFactory<ViewLoadingComponent> = this.componentFactoryResolver.resolveComponentFactory(ViewLoadingComponent);
        const ele: HTMLElement = this.elementRef.nativeElement;
        this.component = componentComponentFactory.create(this.injector);
        this.component.instance.size = this.size;
        const zIndex = this.getElementZIndex(this.elementRef.nativeElement) + 1;
        this.component.instance.zIndex = (zIndex > 1000 ? zIndex : 1000).toString();
        if (getComputedStyle(ele).position === 'static') {
          ele.style.position = 'relative';
        }
        ele.appendChild(this.component.location.nativeElement);
        this.applicationRef.attachView(this.component.hostView);
        this.delayTimer = null;
      }, this.delay);
    } else {
      if (this.delayTimer) {
        clearTimeout(this.delayTimer);
        this.delayTimer = null;
      }
      if (this.component) {
        this.component.destroy();
        this.component = null;
      }
    }
  }

  /**
   * 计算htmlindex
   * @param ele
   * @private
   */
  private getElementZIndex(ele: HTMLElement): number {
    if (getComputedStyle(ele).zIndex !== 'auto') {
      return Number.parseInt(getComputedStyle(ele).zIndex, 10);
    } else if (ele.tagName !== 'BODY') {
      return this.getElementZIndex(ele.parentElement);
    }
    return 0;
  }
}
