export class CookieUtil {
  private pluses = /\+/g;
  private defaults = {};
  private raw: boolean;
  private json: boolean;

  cookie(key, value?: any, options?: any): any {

    if (arguments.length > 1 && typeof value !== 'function') {
      options = Object.assign({}, this.defaults, options);

      if (typeof options.expires === 'number') {
        const seconds = options.expires;
        const t = options.expires = new Date();
        t.setMilliseconds(t.getMilliseconds() + seconds * 1000);
      }
      return (document.cookie = [
        this.encode(key), '=', this.stringifyCookieValue(value),
        options.expires ? '; expires=' + options.expires.toUTCString() : '',
        options.path ? '; path=' + options.path : '',
        options.domain ? '; domain=' + options.domain : '',
        options.secure ? '; secure' : ''
      ].join(''));
    }

    let result = key ? undefined : {};
    let i = 0;
    const cookies = document.cookie ? document.cookie.split('; ') : [];
    const l = cookies.length;

    for (; i < l; i++) {
      const parts = cookies[i].split('=');
      const name = this.decode(parts.shift());
      let cookie = parts.join('=');

      if (key === name) {
        result = this.read(cookie, value);
        break;
      }

      if (!key && (cookie = this.read(cookie)) !== undefined) {
        result[name] = cookie;
      }
    }

    return result;
  }

  removeCookie(key, options = {}): boolean {
    this.cookie(key, '', Object.assign({}, options, {expires: -1}));
    return !this.cookie(key);
  }

  private read(s, converter?: any) {
    const value = this.raw ? s : this.parseCookieValue(s);
    return typeof converter === 'function' ? converter(value) : value;
  }

  private stringifyCookieValue(value) {
    return this.encode(this.json ? JSON.stringify(value) : String(value));
  }

  private parseCookieValue(s: string) {
    if (s.indexOf('"') === 0) {
      s = s.slice(1, -1).replace(/\\"/g, '"').replace(/\\\\/g, '\\');
    }

    try {
      s = decodeURIComponent(s.replace(this.pluses, ' '));
      return this.json ? JSON.parse(s) : s;
    } catch (e) {
    }
  }

  private decode(s: string) {
    return this.raw ? s : decodeURIComponent(s);
  }

  private encode(s: string) {
    return this.raw ? s : encodeURIComponent(s);
  }
}
