import {HttpClient, HttpHeaders, HttpParams} from '@angular/common/http';
import {Injectable} from '@angular/core';
import {ResponseError} from '../domains/response-error.domain';
import {catchError} from 'rxjs/operators';
import {ErrorHandle} from './error-handle';
import {Util} from './util';

@Injectable({
  providedIn: 'root'
})
export class Request {
  private static base = './api/';

  private path: string;

  constructor(
    private http: HttpClient
  ) {
    this.setPath(Request.base);
  }

  static setBase(value: string): void {
    Request.base = value;
  }

  setPath(value: string): void {
    this.path = value;
  }

  getPath(): string {
    return this.path;
  }

  post<T>(url: string, httpPrams: HttpParams | FormData, httpHeaders?: HttpHeaders): Promise<T> {
    const headers = new HttpHeaders().append('Urlencode-Type', 'PS1801');
    if (httpHeaders) {
      const keys = httpHeaders.keys();
      for (const key of keys) {
        headers.append(key, httpHeaders.get(key));
      }
    }
    return new Promise<T>((resolve, reject: (reason: ResponseError) => void) => {
      this.http.post<T>(
        Util.addUrlParam(this.path + url, '_=' + new Date().getTime().toString()),
        httpPrams,
        {
          headers
        }
      ).pipe(
        catchError(ErrorHandle.listen<T>(url))
      ).subscribe(res => {
        if (res === null) {
          resolve();
          return;
        }
        if (typeof res === 'object') {
          if (res.hasOwnProperty('error_response')) {
            reject(new ResponseError(res['error_response']));
            return;
          }
          if (res.hasOwnProperty('Stacks') && res.hasOwnProperty('Code') && res.hasOwnProperty('Msg')) {
            reject(new ResponseError(res));
            return;
          }
        }
        resolve(res as T);
      }, (err: ResponseError) => {
        reject(err);
      });
    });
  }

  file(url: string, httpParams: HttpParams): string {
    const keys = httpParams.keys();
    let requestUrl = Util.addUrlParam(this.path + url, '_=' + new Date().getTime().toString());
    for (const key of keys) {
      requestUrl = Util.addUrlParam(requestUrl, `${key}=${encodeURIComponent(httpParams.get(key))}`);
    }
    return requestUrl;
  }
}
